/** @file ledbox.h

	HITACHI MB-S1 Emulator 'EmuB-S1'
	Skelton for retropc emulator

	@author Sasaji
	@date   2011.08.08 -

	@brief [ led box ]
*/

#ifndef _LEDBOX_H_
#define _LEDBOX_H_

#include <windows.h>

#include <d3d9.h>
#pragma comment(lib, "d3d9.lib")
#ifdef USE_DIRECT3DX
#include <d3dx9.h>
#pragma comment(lib, "d3dx9.lib")
#endif

#include "../common.h"
#include "../dibsection.h"

/**
	@brief LedBox is the class that display the access indicator on the screen.
*/
class LedBoxBase : public DibSection
{
protected:
	HINSTANCE	hInstance;
	uint64		flag_old;

	enum enum_led_type {
		LED_TYPE_VR = 0,
		LED_TYPE_VG,
		LED_TYPE_HR,
		LED_TYPE_HG,
		LED_TYPE_RR,
		LED_TYPE_CL,
		LCD_TYPE_TAPE,
		LCD_TYPE_FDD0,
		LCD_TYPE_FDD1,
		LCD_TYPE_FDD2,
		LCD_TYPE_FDD3,
		LCD_TYPE_3_FDD35H,
		LCD_TYPE_5_FDD35H,
		LCD_TYPE_H_FDD35H,
		LCD_TYPE_AB_A,
		LCD_TYPE_AB_B,
		LCD_TYPE_12M_1,
		LCD_TYPE_12M_2,
		LCD_TYPE_SU_S,
		LCD_TYPE_SU_U,
		LED_TYPE_DIGIT,
		LED_TYPE_PARTS,
		LED_TYPE_BASE,
		LED_TYPE_END
	};

	// LED
	typedef struct led_st {
		HBITMAP     hBmp;
		HDC         hDC;
		SIZE        sz;
	} led_t;
	led_t       led[LED_TYPE_END];

	enum enum_led_pos {
		LED_POS_BASE = 0,
		LED_POS_KATAKANA,
		LED_POS_HIRAGANA,
		LED_POS_CAPS,
		LED_POS_PLAY,
		LED_POS_REC,
		LCD_POS_PLAY,
		LCD_POS_REC,
		LED_POS_FDD0,
		LED_POS_FDD1,
		LED_POS_FDD2,
		LED_POS_FDD3,
		LCD_POS_FDD0,
		LCD_POS_FDD1,
		LCD_POS_FDD2,
		LCD_POS_FDD3,
		LCD_POS_FDD35H,
		LCD_POS_AB,
		LCD_POS_12M,
		LCD_POS_SU,
		LED_POS_DIGIT1,
		LED_POS_DIGIT2,
		LED_POS_DIGIT3,
		LED_POS_END
	};
	// LEDʒu
	typedef struct led_pt_st {
		POINT pt;
		SIZE  sz;
	} led_pt_t;
	led_pt_t	led_pt[LED_POS_END];

	// \ʒu
	struct win_pt_st {
		int left;
		int top;
		int right;
		int bottom;
		int place;
	} win_pt;
	VmRectWH    parent_pt;

	int			mode;
	int         prev_mode;
	VmPoint		dist_set[2];
	VmPoint		dist;

	bool        visible;
	bool        inside;

	bool create_bitmap(HDC hdc, led_t *l, DWORD id);
	bool create_parts(HDC hdc, const led_t &srcl, int x, int y, int w, int h, led_t *l);
	void copy_led(led_pt_t &, led_t &);
	void copy_led_s(led_pt_t &, led_t &, POINT &);

	/* for dialog box */
	virtual void show_dialog() {}
	virtual void move_in_place(int place) {}
	virtual void need_update_dialog() {}
public:
	LedBoxBase();
	virtual ~LedBoxBase();
	bool InitScreen(HDC hdc);
	void Show(int);
	void SetMode(int);
	void SetPos(int left, int top, int right, int bottom, int place);
	void SetPos(int place);
	void Draw(HDC);
	void Draw(LPDIRECT3DSURFACE9);
	void Update(uint64 flag);

	void SetDistance(int place, const VmPoint *ndist);
	void GetDistance(VmPoint *ndist);

	/* for dialog box */
	virtual void CreateDialogBox() {}
	virtual void Move() {}
};

#ifdef _WIN32
#include "windows/win_ledbox.h"
#endif

#endif /* _LEDBOX_H_ */

