/** @file kanji.h

	HITACHI MB-S1 Emulator 'EmuB-S1'
	Skelton for retropc emulator

	@author Sasaji
	@date   2011.10.19 -

	@brief [ kanji ]
*/

#ifndef _KANJI_H_
#define _KANJI_H_

#include "../vm.h"
#include "../../emu.h"
#include "../device.h"

/**
	@brief kanji ROM
*/
class KANJI : public DEVICE
{
private:
	uint16 code;
	uint8  offset;

	uint8 font[0x20000];

	bool font_enable;

	//for resume
#pragma pack(1)
	struct vm_state_st {
		uint16 code;
		uint8  offset;

		char  reserved[13];
	};
#pragma pack()

public:
	KANJI(VM* parent_vm, EMU* parent_emu, const char* identifier) : DEVICE(parent_vm, parent_emu, identifier)
	{
		set_class_name("KANJI");
	}
	~KANJI() {}

	// common functions
	void initialize();
	void reset();
	void write_signal(int id, uint32 data, uint32 mask);
	void event_frame();

	void write_io8(uint32 addr, uint32 data);
	uint32 read_io8(uint32 addr);

	void event_callback(int event_id, int err);

	void save_state(FILEIO *fio);
	void load_state(FILEIO *fio);

#ifdef USE_DEBUGGER
	uint32 debug_read_io8(uint32 addr);
#endif
};

#endif

