/** @file mc6809dasm.h

	Skelton for retropc emulator

	@author Sasaji
	@date   2015.08.29 -

	@brief [ mc6809 disassembler ]
*/

#ifndef _MC6809DASM_H_
#define _MC6809DASM_H_

#ifndef USE_DEBUGGER
#define DASM_SET_INH()
#define DASM_SET_IMM(a1)
#define DASM_SET_DIR1(a1,a2,a3)
#define DASM_SET_DIR2(a1,a2,a3,a4)
#define DASM_SET_IDX1(a1)
#define DASM_SET_IDX2(a1)
#define DASM_SET_EXT1(a1,a2)
#define DASM_SET_EXT2(a1,a2,a3)
#define DASM_SET_JMP(a1)
#define DASM_SET_REL(a1)
#define DASM_SET_TFR(a1)
#define DASM_SET_PSH(a1)
#define DASM_SET_SWI()
#define DASM_SET_SWI2()
#define DASM_SET_SWI3()
#define DASM_SET_ERR()
#define DASM_SET_CODE2(a1)
#define DASM_SET_MEM1(a1,a2)
#define DASM_SET_MEM2(a1,a2,a3)

#else /* USE_DEBUGGER*/
#define DASM_SET_INH()				dasm.set_inh()
#define DASM_SET_IMM(a1)			dasm.set_imm(a1)
#define DASM_SET_DIR1(a1,a2,a3)		dasm.set_dir(a1, a2, a3)
#define DASM_SET_DIR2(a1,a2,a3,a4)	dasm.set_dir(a1, a2, a3, a4)
#define DASM_SET_IDX1(a1)			dasm.set_idx1(a1)
#define DASM_SET_IDX2(a1)			dasm.set_idx2(a1)
#define DASM_SET_EXT1(a1,a2)		dasm.set_ext(a1, a2)
#define DASM_SET_EXT2(a1,a2,a3)		dasm.set_ext(a1, a2, a3)
#define DASM_SET_JMP(a1)			dasm.set_jmp(a1)
#define DASM_SET_REL(a1)			dasm.set_rel(a1)
#define DASM_SET_TFR(a1)			dasm.set_tfr(a1)
#define DASM_SET_PSH(a1)			dasm.set_psh(a1)
#define DASM_SET_SWI()				dasm.set_swi()
#define DASM_SET_SWI2()				dasm.set_swi2()
#define DASM_SET_SWI3()				dasm.set_swi3()
#define DASM_SET_ERR()				dasm.set_err()
#define DASM_SET_CODE2(a1)			dasm.set_code2(a1)
#define DASM_SET_MEM1(a1,a2)		dasm.set_mem(a1, a2)
#define DASM_SET_MEM2(a1,a2,a3)		dasm.set_mem(a1, a2, a3)


#include "../emu.h"
#include "vm.h"
#include "../common.h"

#define MC6809DASM_PCSTACK_COUNT 20

enum en_opmode {
	NONE = 0,
	INH,
	DIR,
	EXT,
	IMM1,
	IMM2,
	IDX,
	REL1,
	REL2,
	JMP,
	TFR,
	PSH,
	SWI2,
	FDB,
	ERR,
};

typedef struct opcode_st {
	uint8         c;
	const _TCHAR *s;
	en_opmode     m;
} opcode_t;

typedef struct regs_st {
	uint32 phyaddr;

	uint16 pc;
	uint16 s;

	uint16 u;
	uint16 x;

	uint16 y;
	uint8  dp;
	uint8  a;

	uint8  b;
	uint8  cc;

	uint8  code[5];

	uint8  valid;
} regs_t;

#define PCSTACK_NUM 64

class MC6809DASM
{
private:
	DEVICE *d_mem;

	_TCHAR line[_MAX_PATH];

	regs_t regs[PCSTACK_NUM];

//	uint32 inphyaddr;
//	uint16 inpc;
//	uint8  inopcode[8];
	int    inopcodelen;

	int    start_accum;
	int    expended_cycles;
	struct st_memdata {
		uint32 phyaddr;
		uint16 addr;
		uint8  size;
		uint16 data;
		bool   write;
	} memdata;

//	uint8  indp;
//	uint8  incc;
//	uint8  ina;
//	uint8  inb;
//	uint16 inx;
//	uint16 iny;
//	uint16 ins;
//	uint16 inu;

	_TCHAR cmd[128];

	uint16 now_halt;
	uint16 now_reset;
	uint16 now_nmi;
	uint16 now_irq;
	uint16 now_firq;

	int set_cmd_str(uint32 addr, const uint8 *ops, en_opmode *mod);
	int set_idx_str(uint16 pc, const uint8 *ops, int opspos);
	int set_tfr_str(const uint8 *ops, int opspos);
	int set_psh_str(const uint8 *ops, int opspos);
	int set_swi2_str(uint32 addr, const uint8 *ops, int opspos);
	uint32 swi2_appear;

	void set_ext_addr(uint16 addr);
	void set_dir_addr(uint8 dp, uint8 addrl);

	void push_stack_pc(uint32 phyaddr, uint16 pc, uint8 code);

public:
	MC6809DASM();
	~MC6809DASM();

	void ini_pc(uint16 pc, uint8 code);
	void set_code2(uint8 code2);
	void set_swi2_param(uint8 cat, uint8 idx);

	void set_mem(uint16 addr, uint8 data, bool write = false);
	void set_mem(uint16 addr, uint16 data, bool write = false);
	void set_phymem(uint32 addr, uint8 data, bool write = false);

	void set_inh();
	void set_swi();
	void set_swi2();
	void set_swi3();

	void set_dir(uint8 dp, uint8 addrl, uint8 data, bool write = false);
	void set_dir(uint8 dp, uint8 addrl, uint16 data, bool write = false);

	void set_imm(uint8 data);
	void set_imm(uint16 data);

	void set_ext(uint16 addr, uint8 data, bool write = false);
	void set_ext(uint16 addr, uint16 data, bool write = false);

	void set_idx1(uint8 data);
	void set_idx2(uint8 data);
	void set_idx2(uint16 data);

	void set_rel(uint8 data);
	void set_rel(uint16 data);
	void set_jmp(uint16 data);

	void set_tfr(uint8 data);

	void set_psh(uint8 data);

	void set_err();

	void set_signal(int id, uint32 data, int accum);

	void set_regs(int accum, int cycles, uint8 cc, uint8 dp, uint8 a, uint8 b, uint16 x, uint16 y, uint16 s, uint16 u);

	int print_dasm(uint32 phy_addr, uint16 pc, const uint8 *ops, int opslen, uint8 *dp = NULL);
	int print_dasm_preprocess(uint32 phy_addr, uint16 pc, const uint8 *ops, int opslen, uint8 *dp = NULL);
	int print_dasm_processed(uint16 pc);

	void print_cycles(int cycles);
	void print_regs(uint8 cc, uint8 dp, uint8 a, uint8 b, uint16 x, uint16 y, uint16 s, uint16 u);
	void print_memory_data();

	int get_stack_pc(int index, regs_t *stack);

	const _TCHAR *get_line() const {
		return line;
	}
	size_t get_line_length() const {
		return _tcslen(line);
	}

	void set_context_mem(DEVICE* device) {
		d_mem = device;
	}
};

#endif /* USE_DEBUGGER */

#endif /* _MC6809DASM_H_ */
