/** @file win32_input_keysym.cpp

	Skelton for retropc emulator

	@author Sasaji
	@date   2014.04.01

	@brief [ win32 input keysym ]
*/

#include "emu.h"
#include "vm/vm.h"
#include "gui/gui.h"
#include "keycode.h"

/// notify accel key
int EMU::system_key_down(int code)
{
	if (vm) {
		code = translate_global_key(code);
		vm->system_key_down(code);
	}
	return code;
}
void EMU::system_key_up(int code)
{
	if (vm) {
		code = translate_global_key(code);
		vm->system_key_up(code);
	}
}

/// execute accel key
bool EMU::execute_global_keys(int code, uint32 status)
{
	if (gui) {
		code = translate_global_key(code);
		return gui->ExecuteGlobalKeys(code, status);
	}
	return false;
}

/// release accel key
bool EMU::release_global_keys(int code, uint32 status)
{
	if (gui) {
		code = translate_global_key(code);
		return gui->ReleaseGlobalKeys(code, status);
	}
	return false;
}

/// translate
int EMU::translate_global_key(int code)
{
	if (code == VK_CONTROL) {
		code = GLOBALKEY_CONTROL;
	} else if (code == VK_RETURN) {
		code = GLOBALKEY_RETURN;
	} else if (code == '0') {
		code = GLOBALKEY_0;
	} else if ('1' <= code && code <= '9') {
		code = code + GLOBALKEY_1 - '1';
	} else if ('A' <= code && code <= 'Z') {
		code = code + GLOBALKEY_A - 'A';
	} else if (VK_F1 <= code && code <= VK_F12) {
		code = code + GLOBALKEY_F1 - VK_F1;
	} else if (VK_F13 <= code && code <= VK_F15) {
		code = code + GLOBALKEY_F13 - VK_F13;
	}
	return code;
}

/// post message to main thread
void EMU::post_command_message(int id)
{
	if (gui) gui->PostCommandMessage(id);
}